import { Component } from '@angular/core';
import {DatePipe, NgForOf, NgOptimizedImage, NgStyle} from "@angular/common";
import {NzButtonComponent} from "ng-zorro-antd/button";
import {NzColDirective, NzRowDirective} from "ng-zorro-antd/grid";
import {NzFormControlComponent, NzFormDirective, NzFormItemComponent, NzFormLabelComponent} from "ng-zorro-antd/form";
import {NzInputDirective} from "ng-zorro-antd/input";
import {NzOptionComponent, NzSelectComponent} from "ng-zorro-antd/select";
import {FormGroup, NonNullableFormBuilder, ReactiveFormsModule, Validators} from "@angular/forms";
import {NzMessageService} from "ng-zorro-antd/message";
import {NzModalComponent, NzModalContentDirective, NzModalModule} from "ng-zorro-antd/modal";
import {GoodsService} from "../../../services/goods/goods.service";
import {NzIconDirective} from "ng-zorro-antd/icon";
import {NzTreeSelectComponent} from "ng-zorro-antd/tree-select";
import {NzUploadComponent, NzUploadFile} from "ng-zorro-antd/upload";
import {environment} from "../../../../environments/environment";
import {GoodsTypeData} from "../../../model/goods.interface";
import {CommonService} from "../../../services/common/common.service";
import {NzDividerComponent} from "ng-zorro-antd/divider";
import {
  NzTableCellDirective,
  NzTableComponent,
  NzTbodyComponent,
  NzTheadComponent,
  NzThMeasureDirective, NzTrDirective
} from "ng-zorro-antd/table";
import {NzPopconfirmDirective} from "ng-zorro-antd/popconfirm";
import {fallbackImg} from "../../../data/goods/goods.namespace";
import {NzImageModule} from "ng-zorro-antd/image";

const getBase64 = (file: File): Promise<string | ArrayBuffer | null> =>
  new Promise((resolve, reject) => {
    const reader = new FileReader();
    reader.readAsDataURL(file);
    reader.onload = () => resolve(reader.result);
    reader.onerror = error => reject(error);
  });

@Component({
  selector: 'app-goods-brand',
  standalone: true,
  imports: [
    NgForOf,
    NzButtonComponent,
    NzColDirective,
    NzFormControlComponent,
    NzFormDirective,
    NzFormItemComponent,
    NzFormLabelComponent,
    NzInputDirective,
    NzOptionComponent,
    NzRowDirective,
    NzModalModule,
    NzSelectComponent,
    ReactiveFormsModule,
    NzIconDirective,
    NzModalComponent,
    NzTreeSelectComponent,
    NzUploadComponent,
    NzModalContentDirective,
    NgStyle,
    DatePipe,
    NzDividerComponent,
    NzTableCellDirective,
    NzTableComponent,
    NzTbodyComponent,
    NzThMeasureDirective,
    NzTheadComponent,
    NzTrDirective,
    NzPopconfirmDirective,
    NzImageModule,
    NgOptimizedImage
  ],
  templateUrl: './goods-brand.component.html',
  styleUrl: './goods-brand.component.less'
})
export class GoodsBrandComponent {
  // 表单页数
  tablePageNum = 1;
  // 表单数据
  tableData: any = {
    total: 0,
    loading: false,
    list: [],
  };
  // 搜索表单
  searchForm: FormGroup;

  // 图片
  imgUrl: NzUploadFile[] = []
  baseUrl = environment.baseUrl;
  imageUrl = environment.imageUrl;

  // 展示图片
  previewImage: string | undefined = '';
  // 上传是否展示
  previewVisible = false;

  // 编辑产品类型表单
  editForm: FormGroup;
  // 编辑弹出框
  isVisibleEdit = false;

  protected readonly fallbackImg = fallbackImg;

  constructor(private fb: NonNullableFormBuilder,
              private msg: NzMessageService,
              private commonService: CommonService,
              private goodsService: GoodsService) {
    // 初始化搜索框
    this.searchForm = this.fb.group({
      title: [''],
    });

    // 初始化
    this.editForm = this.fb.group({
      title: ['' , [Validators.required]],
      id: [null],
      imgUrl: [''],
    });
    this.getRequest();

  }

  // 查询列表
  public getRequest(reset: boolean = false) {
    this.tableData.loading  = true;
    if (reset) {
      this.tablePageNum = 1;
    }
    this.searchForm.value.pageNum = this.tablePageNum;
    this.searchForm.value.pageSize = 10;
    this.searchForm.value.time = new Date().getTime();

    this.goodsService.getListBrand(this.searchForm.value , (data: any) => {
      if (data['return_code'] == '000000') {
        this.tableData = data['return_data'];
      } else {
        this.msg.error(data['return_msg']);
      }
      this.tableData.loading  = false;
    });
  }


  // 唤醒编辑表单
  showEdit(item?: GoodsTypeData) {
    if (item != null) {
      item.businessType = String(item.businessType);
      this.editForm.patchValue(item);
      if (!this.commonService.whetherStringIsNull(item.imgUrl)) {
        this.imgUrl = this.commonService.stitchImg(item.imgUrl);
      }
    } else {
      this.editForm.reset();
    }
    this.isVisibleEdit = true;
  }
  // 编辑表单提交
  handleEdit(): void {

    if (this.editForm.valid) {
      if (this.imgUrl.length !== 0) {
        this.editForm.value.imgUrl = this.commonService.imgList(this.imgUrl);
      }
      this.goodsService.editGoodsBrand(this.editForm.value , (data: any) => {
        if (data['return_code'] === '000000') {
          this.msg.success("成功！");
          this.isVisibleEdit = false;
          this.getRequest();
        } else {
          this.msg.error(data['return_msg']);
        }
      });
    } else {
      Object.values(this.editForm.controls).forEach(control => {
        if (control.invalid) {
          control.markAsDirty();
          control.updateValueAndValidity({ onlySelf: true });
        }
      });
    }

  }

  // 上传操作
  handlePreview = async (file: NzUploadFile): Promise<void> => {
    if (!file.url && !file['preview']) {
      file['preview'] = await getBase64(file.originFileObj!);
    }
    this.previewImage = file.url || file['preview'];
    this.previewVisible = true;
  };

  // 删除
  delete(id: number): void {
    this.goodsService.deleteBrand(id , (data: any) => {
      if (data['return_code'] === '000000') {
        this.msg.success("成功！");
        this.getRequest();
      } else {
        this.msg.error(data['return_msg']);
      }
    });
  }

}
